/*
 * Copyright (C) 2014  David Edmundson <david@davidedmundson.co.uk>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 *
 */

#ifndef ABSTRACTNOTELOADER_H
#define ABSTRACTNOTELOADER_H

class QStringList;
class QString;
class Note;

class AbstractNoteLoader
{
public:
    explicit AbstractNoteLoader();
    virtual ~AbstractNoteLoader();

    virtual QStringList allNoteIds() = 0;
    virtual Note* loadNote(const QString &id) = 0;
    virtual void  deleteNoteResources(const QString &id) = 0;
private:
};


#endif // ABSTRACTNOTEMANAGER_H
